module sintr_mod
contains
subroutine sintr(f, msign, h)

! *******************************************************************
! *                                                                 *
! * integral sine transform of a real function f(x)                 *
! *                                                                 *
! * g(y) = integral from zero to infinity of f(x)*sin(x*y) dx       *
! * if msign >= 0 or                                                *
! * g(y) = y * integral from zero to infinity of f(x)*sin(x*y) dx   *
! * if msign < 0                                                    *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**iabs(msig)                      *
! * g(y) is computed on the mesh of points yk = k*2pi/(n*h) ,       *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * msign is such that n = 2**iabs(msign)                           *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(yk), k = 0, 1, ..., n-1                       *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! *                                                                 *
! *******************************************************************

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: msign
  double precision, intent(in) :: h

  double precision :: a, c, ca, d, e, fnp1, s, sa, ti, ti1, ti2, tr, tr1, tr2
  double precision :: ui, ur, wi, wr
  integer :: i, i2, ip2, j, j2, k, l, le, le1, m2, n, n2, nm1, nv2

  logical :: debug

  debug = .false.
  if (h <= 0.0d0) then
    if (debug) write(*,*) ' *** incorrect step size in <sintr>, h: ', h, ' ***'
    stop
  endif

  if (msign == 0) then
    f(1) = 0.0d0
    return
  endif

! *** for j and k = 0, 1, 2 ... n/2-1, compute
! s1 = sum ( f(2*j+1)*cos(4*pi*k*j/n) - f(2*j+2)*sin(4*pi*k*j/n) )
! s2 = sum ( f(2*j+1)*sin(4*pi*k*j/n) + f(2*j+2)*cos(4*pi*k*j/n) )
! store s1 in f(2*k+1) and s2 in f(2*k+2)
! (adapted from cfft routine, cern library member d704)

  m2 = iabs(msign) - 1
  n = 2**m2
  if (n /= 1) then
    nv2 = n / 2
    nm1 = n - 1
    j = 1
    do i = 1, nm1
      if (i < j) then
        i2 = i + i
        j2 = j + j
        ti = f(j2)
        f(j2) = f(i2)
        f(i2) = ti
        i2 = i2 - 1
        j2 = j2 - 1
        tr = f(j2)
        f(j2) = f(i2)
        f(i2) = tr
      endif
      k = nv2
      do while (j > k)
        j = j - k
        k = k / 2
      enddo
      j = j + k
    enddo
    do i = 1, n, 2
      i2 = i + i
      ti = f(i2 + 2)
      f(i2 + 2) = f(i2) - ti
      f(i2) = f(i2) + ti
      i2 = i2 - 1
      tr = f(i2 + 2)
      f(i2 + 2) = f(i2) - tr
      f(i2) = f(i2) + tr
    enddo
    if (m2 /= 1) then
      c = 0.0d0
      s = 1.0d0
      le = 2
      do l = 2, m2
        wr = c
        wi = s
        ur = wr
        ui = wi
        c = dsqrt(c * 0.5d0 + 0.5d0 )
        s = s / (c + c)
        le1 = le
        le = le1 + le1
        do i = 1, n, le
          i2 = i + i
          ip2 = i2 + le
          ti = f(ip2)
          f(ip2) = f(i2) - ti
          f(i2) = f(i2) + ti
          i2 = i2 - 1
          ip2 = ip2 - 1
          tr = f(ip2)
          f(ip2) = f(i2) - tr
          f(i2) = f(i2) + tr
        enddo
        do j = 2, le1
          do i = j, n, le
            i2 = i + i
            ip2 = i2 + le
            tr = f(ip2 - 1) * ur - f(ip2) * ui
            ti = f(ip2) * ur + f(ip2 - 1) * ui
            f(ip2) = f(i2) - ti
            f(i2) = f(i2) + ti
            i2 = i2 - 1
            ip2 = ip2 - 1
            f(ip2) = f(i2) - tr
            f(i2) = f(i2) + tr
          enddo
          tr = ur * wr - ui * wi
          ui = ui * wr + ur * wi
          ur = tr
        enddo
      enddo
    endif
  endif

! *** for j and k = 0, 1 ... n-1, transform the array f so obtained into
! sum f(j+1)*cos(2*pi*k*j/n) and sum f(j+1)*sin(2*pi*k*j/n)
! and multiply the results by (1 - cos(2*pi*k/n)) and sin(2*pi*k/n)

  fnp1 = 4 * (f(1) - f(2))
  a = 3.141592653589793238d0 / n
  n2 = n + 1
  n = 2 * n
  if (n2 /= 2) then
    c = 1.0d0
    s = 0.0d0
    ca = dcos(a)
    sa = dsin(a)
    k = n + 1
    do j = 3, n2, 2
      k = k - 2
      d = c
      c = d * ca - s * sa
      s = d * sa + s * ca
      tr1 = f(j) + f(k)
      ti1 = f(j + 1) - f(k + 1)
      d = f(j) - f(k)
      e = f(j + 1) + f(k + 1)
      tr2 = d * s + e * c
      ti2 = e * s - d * c
      f(j) = (1.0d0 - c) * (tr1 + tr2)
      f(j + 1) = s * (ti1 + ti2)
      f(k) = (1.0d0 + c) * (tr1 - tr2)
      f(k + 1) = s * (ti2 - ti1)
    enddo
    n2 = n2 - 1
    do j = 2, n2
      j2 = j + j
      f(j) = f(j2 - 1) + f(j2)
    enddo
    do j = 2, n2
      f(n + 2 - j) = f(j)
    enddo
    n2 = n2 + 1
  endif
  f(n2) = fnp1
  if (msign >= 0) then

! *** normalization

    c = 0.0d0
    a = (a + a) / h
    do j = 2, n
      c = c + a
      f(j) = f(j) / c
    enddo
  endif
  f(1) = 0.0d0
  return
end subroutine sintr
end module sintr_mod
